@@ -57,6 +57,17 @@ class CloudFlow:
         else:
             self.auth_header = {'Authorization': f'token {token}'}
 
+        if self.path is not None and not Path(self.path).exists():
+            _exit_error(f'The path {self.path} specificed doesn\'t exist.')
+
+        if self.env_file is not None:
+            if (
+                not Path(self.env_file).exists()
+                or not Path(self.env_file).is_file()
+                or Path(self.env_file).suffix != '.env'
+            ):
+                _exit_error('The env_file specified isn\'t a valid .env file.')
+
         if self.flow_id and not self.flow_id.startswith('jflow-'):
             # user given id does not starts with `jflow-`
             self.flow_id = f'jflow-{self.flow_id}'
@@ -82,11 +93,7 @@ class CloudFlow:
         _tags = {'filename': _path.name if _path.is_file() else 'flow.yml'}
         if self.env_file is not None:
             _env_path = Path(self.env_file)
-            if _env_path.exists():
-                logger.info(f'Passing env variables from {self.env_file} file')
-                _tags.update({'envfile': _env_path.name})
-            else:
-                _exit_error(f'env file [b]{self.env_file}[/b] not found')
+            _tags.update({'envfile': _env_path.name})
         else:
             _env_path = _path / '.env'
             if _env_path.exists():
@@ -98,12 +105,9 @@ class CloudFlow:
     def envs(self) -> Dict:
         if self.env_file is not None:
             _env_path = Path(self.env_file)
-            if _env_path.exists():
-                from dotenv import dotenv_values
+            from dotenv import dotenv_values
 
-                return dict(dotenv_values(_env_path))
-            else:
-                _exit_error(f'env file [b]{self.env_file}[/b] not found')
+            return dict(dotenv_values(_env_path))
         else:
             return {}
 
@@ -125,16 +129,15 @@ class CloudFlow:
 
         _data = aiohttp.FormData()
         _path = Path(self.path)
-        if not _path.exists():
-            _exit_error(f'Path {self.path} doesn\'t exist.')
-        elif _path.is_dir():
+
+        if _path.is_dir():
             _flow_path = _path / 'flow.yml'
-            if _flow_path.exists() and normalized(_flow_path):
+            if _flow_path.exists() and normalized(_flow_path, self.envs):
                 _data.add_field(name='yaml', value=open(_flow_path))
             else:
                 params['artifactid'] = await self._zip_and_upload(_path)
         elif _path.is_file():
-            if normalized(_path):
+            if normalized(_path, self.envs):
                 _data.add_field(name='yaml', value=open(_path))
             else:
                 # normalize & deploy parent directory
@@ -302,11 +305,17 @@ class CloudFlow:
         def dim_print(text):
             print(f'[dim]{text}[/dim]')
 
-        log_msg = dim_print if 'request_id' in params else print
+        _log_fn = dim_print if 'request_id' in params else print
+        _skip_debug_logs = (
+            'request_id' in params and logger.getEffectiveLevel() >= logging.INFO
+        )
 
         try:
             async with aiohttp.ClientSession() as session:
-                while True:
+                _num_retries = 3
+                for i in range(_num_retries):
+                    # NOTE: Websocket endpoint has a default timeout of 15mins, after which connection drops.
+                    # We'll retry the connection `_num_retries` times.
                     try:
                         async with session.ws_connect(
                             LOGSTREAM_API, params=params
@@ -318,17 +327,16 @@ class CloudFlow:
                             async for msg in ws:
                                 if msg.type == aiohttp.http.WSMsgType.TEXT:
                                     log_dict: Dict = msg.json()
-                                    if (
-                                        log_dict.get('status') == 'STREAMING'
-                                        and logger.getEffectiveLevel() < logging.INFO
-                                    ):
-                                        log_msg(log_dict['message'])
-                        if 'request_id' in params:
-                            # NOTE: For the case of `request_id`, Flow deployment might be taking >15mins,
-                            # after which websocket endpoint will timeout. Hence reconnect & stream.
-                            continue
-                        else:
-                            logger.debug(f'Disconnected from the logstream server ...')
+                                    if log_dict.get('status') == 'STREAMING':
+                                        if _skip_debug_logs:
+                                            continue
+                                        _log_fn(log_dict['message'])
+                        logger.debug(
+                            f'Disconnected from the logstream server ... '
+                            + 'Retrying ..'
+                            if i <= _num_retries
+                            else ''
+                        )
                     except aiohttp.WSServerHandshakeError as e:
                         logger.critical(
                             f'Couldn\'t connect to the logstream server as {e!r}'
@@ -345,7 +353,7 @@ class CloudFlow:
                 pb_task,
                 advance=1,
                 description='Submitting',
-                title=f'Deploying {Path(self.path).resolve()}',
+                title=f'Deploying {self.name or Path(self.path).resolve()}',
             )
             await self._deploy()
             pbar.update(pb_task, description='Queueing (can take ~1 minute)', advance=1)
@@ -412,3 +420,23 @@ class CloudFlow:
         my_table.add_row('ID', self.id)
         my_table.add_row('URL', self.gateway)
         yield Panel(my_table, title=':tada: Flow is available!', expand=False)
+
+
+async def _terminate_flow_simplified(flow_id):
+    """Terminate a Flow given flow_id.
+
+    This is a simplified version of CloudFlow.__aexit__, i.e.,
+    without reporting the details of the termination process using the progress bar.
+    It's supposed to be used in the multi-flow removal context.
+    """
+
+    flow = CloudFlow(flow_id=flow_id)
+    await flow._terminate()
+    await flow._fetch_until(
+        intermediate=[Status.SUBMITTED, Status.DELETING],
+        desired=Status.DELETED,
+    )
+    flow._t_logstream_task.cancel()
+
+    # This needs to be returned so in asyncio.as_completed, it can be printed.
+    return flow_id
