@@ -1,65 +1,87 @@
 import asyncio
+import json
 import logging
 import os
 import shutil
 import sys
 import tempfile
+import threading
 import warnings
 from contextlib import contextmanager
+from packaging.version import Version
 from pathlib import Path
-import json
+from typing import Dict, Union
+from urllib.parse import urlparse
+from urllib.request import Request, urlopen
+
+import pkg_resources
+import yaml
+from rich import print
+from rich.highlighter import ReprHighlighter
+from rich.panel import Panel
+
+from .env_helper import EnvironmentVariables, expand_dict
 
 __windows__ = sys.platform == 'win32'
 
 
-def is_latest_version(package):
-
+def _version_check(package: str = None, github_repo: str = None):
     try:
-        import warnings
-        from urllib.request import Request, urlopen
-        import pkg_resources
-        from rich import print
-        from rich.panel import Panel
-        from distutils.version import LooseVersion
 
-        cur_ver = LooseVersion(pkg_resources.get_distribution(package).version)
+        if not package:
+            package = vars(sys.modules[__name__])['__package__']
+        if not github_repo:
+            github_repo = package
+
+        cur_ver = Version(pkg_resources.get_distribution(package).version)
         req = Request(
             f'https://pypi.python.org/pypi/{package}/json',
             headers={'User-Agent': 'Mozilla/5.0'},
         )
         with urlopen(
-            req, timeout=5
+            req, timeout=1
         ) as resp:  # 'with' is important to close the resource after use
             j = json.load(resp)
             releases = j.get('releases', {})
             latest_release_ver = list(
-                sorted(LooseVersion(v) for v in releases.keys() if '.dev' not in v)
+                sorted(Version(v) for v in releases.keys() if '.dev' not in v)
             )[-1]
             if cur_ver < latest_release_ver:
                 print(
                     Panel(
-                        f'You are using [b]{cur_ver}[/b], but [green][b]{latest_release_ver}[/b][/green] is available. '
-                        f'You may upgrade via [b]pip install -U {package}[/b]',
-                        title=':new: New version available',
+                        f'You are using [b]{package} {cur_ver}[/b], but [bold green]{latest_release_ver}[/] is available. '
+                        f'You may upgrade it via [b]pip install -U {package}[/b]. [link=https://github.com/jina-ai/{github_repo}/releases]Read Changelog here[/link].',
+                        title=':new: New version available!',
                         width=50,
                     )
                 )
-    except Exception as ex:
+    except Exception:
         # no network, too slow, PyPi is down
-        get_logger().debug(ex)
+        pass
 
 
-def get_logger():
+def is_latest_version(package: str = None, github_repo: str = None) -> None:
+    """Check if there is a latest version from Pypi, set env `NO_VERSION_CHECK` to disable it.
+
+    :param package: package name if none auto-detected
+    :param github_repo: repo name that contains CHANGELOG if none then the same as package name
+    """
+
+    threading.Thread(target=_version_check, args=(package, github_repo)).start()
+
+
+def get_logger(name='jcloud'):
     from rich.logging import RichHandler
 
-    logging.basicConfig(
-        level=os.environ.get('JCLOUD_LOGLEVEL'),
-        format='%(message)s',
-        datefmt='[%X]',
-        handlers=[RichHandler(rich_tracebacks=True)],
-    )
+    logger = logging.getLogger(name)
+    logger.setLevel(os.environ.get('JCLOUD_LOGLEVEL', 'INFO'))
 
-    return logging.getLogger('jcloud')
+    rich_handler = RichHandler(rich_tracebacks=True)
+    formatter = logging.Formatter('%(message)s')
+    rich_handler.setFormatter(formatter)
+    logger.addHandler(rich_handler)
+
+    return logger
 
 
 def get_or_reuse_loop():
@@ -136,3 +158,40 @@ def zipdir(directory: Path) -> Path:
     shutil.move(_zip_name, _zip_dest)
     yield Path(os.path.join(_zip_dest, os.path.basename(_zip_name)))
     shutil.rmtree(_zip_dest)
+
+
+def valid_uri(uses):
+    try:
+        return urlparse(uses).scheme in (
+            'docker',
+            'jinahub+docker',
+            'jinahub+sandbox',
+            'jinahub',
+        )
+    except Exception:
+        return False
+
+
+def normalized(path: Union[str, Path], envs: Dict):
+    _normalized = True
+    with open(path) as f:
+        _flow_dict = yaml.safe_load(f.read())
+
+    if 'executors' in _flow_dict:
+        with EnvironmentVariables(envs):
+            expand_dict(_flow_dict, context=envs)
+        for executor in _flow_dict['executors']:
+            uses = executor.get('uses', None)
+            if uses is None:
+                continue
+            elif valid_uri(uses):
+                continue
+            else:
+                _normalized = False
+    return _normalized
+
+
+class CustomHighlighter(ReprHighlighter):
+    highlights = ReprHighlighter.highlights + [
+        r"(?P<url>(grpc|grpcs)://[-0-9a-zA-Z$_+!`(),.?/;:&=%#]*)"
+    ]
